#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <gl/gl.h>
#include <dmedia/vl.h> 
#include <sys/time.h>

#define GLOBAL 
#include "global.h"

#define YUV_411_FRAME_SIZE 38016
#define CODE_RATE 10

typedef struct _h261frame
  {
     int size;
     char buf[10000];
  } H261frame;

VLServer svr;
VLPath path;
VLNode src, drn;
VLControlValue val;
VLBuffer buffer;
VLInfoPtr info;
char *dataPtr;
int c;
int xsize;
int ysize;
long win;

unsigned char *frame[3];

struct timeval start_time, stop_time;

H261frame tab[100];

void init_capture();
void close_capture();

void error(char *text)
  {
     printf(text);
  }

/* Report errors */
void
error_exit(void)
{
    vlPerror(""); 
    exit(1);
}

void
main(int argc, char **argv)
{
    FILE *vfile, *afile;
    char buf[YUV_411_FRAME_SIZE];

    char compressed[20000];
    char compressed1[20000];
    char decompressed[38016];
    int size;
    int total=0;

    int i;
    int count = 0;
    int frame_no = 100;
    int code=0;
    int first=1;

    struct timeval start, stop;
    struct timeval start_time, loop_time;
    int loop=1;
    int diff;
    float rate;

    FILE *file;

    init_codec();
    init_decoder();
    init_encoder();

    init_capture();
    init_display("");

    gettimeofday(&start);

    for(i=0; i<frame_no; i++) {
	/* Write the data to the screen */
	/*	lrectwrite(0,0, xsize-1, ysize-1, (ulong *)dataPtr); */
	
	do { 
	   info = vlGetLatestValid(svr, buffer);
	} while (!info); 

	/* Get a pointer to the frame */
	dataPtr = vlGetActiveRegion(svr, buffer, info);
	
	/* YUYV 422 to YUV 411 convert (format for h.263 coder) */
	convert(dataPtr, buf);
	
	/* Finished with frame, unlock the buffer */
	vlPutFree(svr, buffer);

	size = p64EncodeFrame(buf, tab[i].buf);
	total += size;
	tab[i].size = size;

	printf("Frame size: %d\n", size); 
     }
    gettimeofday(&stop);

    for(i=0; i<frame_no; i++)
      {
	 gettimeofday(&start_time);
	 while(loop)
	   {
	      gettimeofday(&loop_time);
	      diff = (loop_time.tv_sec - start_time.tv_sec)*1000 
		 + (loop_time.tv_usec - start_time.tv_usec)/1000;
	      if(diff > 100)
		{
		 loop = 0;
	      }
	   }
	 loop = 1;


	if(first)
	  {
	     decode_frame(tab[i].buf, tab[i].size, decompressed, 1);
	     first = 0;
	  }
	else
	     decode_frame(tab[i].buf, tab[i].size, decompressed, 0);


	frame[0] = decompressed;
	frame[1] = decompressed + 25344;
	frame[2] = decompressed + 25344 + 6336;

	dither(frame);

     }

    diff = (stop.tv_sec - start.tv_sec)*1000 
       + (stop.tv_usec - start.tv_usec)/1000;

    rate = (float)frame_no*1000/(float)diff;

    printf("Rate: %f\n", rate);
    printf("Bytess per frame: %f\n", (float)total/(float)100);
    
    printf("Stop capturing\n");
    close_capture();
}


void init_capture()
  {
    if (!(svr = vlOpenVideo(""))) 
	error_exit();

    drn = vlGetNode(svr, VL_DRN, VL_MEM, VL_ANY);
    
    src = vlGetNode(svr, VL_SRC, VL_VIDEO, VL_ANY);

    path = vlCreatePath(svr, VL_ANY, src, drn); 

    if ((vlSetupPaths(svr, (VLPathList)&path, 1, VL_SHARE, VL_SHARE)) < 0)
	error_exit();

    val.intVal = 10;
    vlSetControl(svr, path, drn, VL_RATE, &val);

    val.fractVal.numerator = 1;
    val.fractVal.denominator = 3;
    vlSetControl(svr, path, drn, VL_ZOOM, &val);

    val.xyVal.x = 176;
    val.xyVal.y = 144;
    vlSetControl(svr, path, drn, VL_SIZE, &val);

    val.intVal = VL_PACKING_YVYU_422_8;
    vlSetControl(svr, path, drn, VL_PACKING, &val);
    
    vlGetControl(svr, path, drn, VL_SIZE, &val);
    xsize = val.xyVal.x;
    ysize = val.xyVal.y;

    vlGetControl(svr, path, drn, VL_RATE, &val);
    printf("Rate: %d\n", val.intVal);


    buffer = vlCreateBuffer(svr, path, drn, 2);
    if (buffer == NULL)
       error_exit();	
    vlRegisterBuffer(svr, path, drn, buffer);
    
    if (vlBeginTransfer(svr, path, 0, NULL))
       error_exit();

    printf("Transfer size: %d\n", vlGetTransferSize(svr, path));
 }

void close_capture()
  {
    vlEndTransfer(svr, path);
     
    vlDeregisterBuffer(svr, path, drn, buffer);
    vlDestroyBuffer(svr, buffer);
    vlDestroyPath(svr, path);
    vlCloseVideo(svr);
  }

